@@ -8,15 +8,14 @@ from orient.hints import (Contour,
                           Multisegment,
                           Point,
                           Segment)
-from . import bounding_box
+from . import bounding
 from .processing import (process_closed_linear_queue,
                          process_open_linear_queue)
 from .relation import Relation
 from .segment import (relate_point as relate_point_to_segment,
                       relate_segment as relate_segments)
-from .sweep import (ClosedSweeper,
-                    OpenSweeper)
-from .utils import flatten
+from .sweep import (CompoundSweeper,
+                    LinearSweeper)
 
 
 def relate_point(contour: Contour, point: Point) -> Relation:
@@ -87,12 +86,12 @@ def relate_multisegment(contour: Contour,
     if not multisegment:
         return Relation.DISJOINT
     contour_bounding_box, multisegment_bounding_box = (
-        bounding_box.from_points(contour),
-        bounding_box.from_points(flatten(multisegment)))
-    if bounding_box.disjoint_with(contour_bounding_box,
+        bounding.box_from_iterable(contour),
+        bounding.box_from_iterables(multisegment))
+    if bounding.box_disjoint_with(contour_bounding_box,
                                   multisegment_bounding_box):
         return Relation.DISJOINT
-    sweeper = OpenSweeper()
+    sweeper = LinearSweeper()
     sweeper.register_segments(to_segments(contour),
                               from_test=False)
     sweeper.register_segments(multisegment,
@@ -104,13 +103,13 @@ def relate_multisegment(contour: Contour,
 
 
 def relate_contour(goal: Contour, test: Contour) -> Relation:
-    goal_bounding_box, test_bounding_box = (bounding_box.from_points(goal),
-                                            bounding_box.from_points(test))
-    if bounding_box.disjoint_with(goal_bounding_box, test_bounding_box):
+    goal_bounding_box, test_bounding_box = (bounding.box_from_iterable(goal),
+                                            bounding.box_from_iterable(test))
+    if bounding.box_disjoint_with(goal_bounding_box, test_bounding_box):
         return Relation.DISJOINT
     if equal(goal, test):
         return Relation.EQUAL
-    sweeper = ClosedSweeper()
+    sweeper = CompoundSweeper()
     sweeper.register_segments(to_segments(goal),
                               from_test=False)
     sweeper.register_segments(to_segments(test),
