@@ -1,3 +1,5 @@
+from typing import Iterable
+
 from orient.hints import (Contour,
                           Multiregion,
                           Multisegment,
@@ -5,19 +7,20 @@ from orient.hints import (Contour,
                           Polygon,
                           Region,
                           Segment)
-from . import bounding_box
+from . import bounding
 from .multiregion import (_relate_contour as relate_contour_to_multiregion,
                           _relate_multisegment
                           as relate_multisegment_to_multiregion,
-                          _relate_region as _relate_region_to_multiregion,
+                          _relate_region as relate_region_to_regions,
                           relate_multiregion as relate_multiregions,
-                          relate_region as relate_region_to_multiregion,
-                          relate_segment as relate_segment_to_multiregion)
+                          relate_segment as relate_segment_to_multiregion,
+                          to_segments as multiregion_to_segments)
 from .region import (_relate_contour as relate_contour_to_region,
                      _relate_multisegment as relate_multisegment_to_region,
                      _relate_region as relate_regions,
                      relate_point as relate_point_to_region,
-                     relate_segment as relate_segment_to_region)
+                     relate_segment as relate_segment_to_region,
+                     to_segments as region_to_segments)
 from .relation import Relation
 
 
@@ -60,14 +63,14 @@ def relate_segment(polygon: Polygon, segment: Segment) -> Relation:
 def relate_multisegment(polygon: Polygon,
                         multisegment: Multisegment) -> Relation:
     return (_relate_multisegment(polygon, multisegment,
-                                 bounding_box.from_iterables(multisegment))
+                                 bounding.box_from_iterables(multisegment))
             if multisegment
             else Relation.DISJOINT)
 
 
 def _relate_multisegment(polygon: Polygon,
                          multisegment: Multisegment,
-                         multisegment_bounding_box: bounding_box.BoundingBox
+                         multisegment_bounding_box: bounding.Box
                          ) -> Relation:
     border, holes = polygon
     relation_without_holes = relate_multisegment_to_region(
@@ -94,7 +97,7 @@ def _relate_multisegment(polygon: Polygon,
 
 def relate_contour(polygon: Polygon, contour: Contour) -> Relation:
     border, holes = polygon
-    contour_bounding_box = bounding_box.from_iterable(contour)
+    contour_bounding_box = bounding.box_from_iterable(contour)
     relation_without_holes = relate_contour_to_region(border, contour,
                                                       contour_bounding_box)
     if holes and (relation_without_holes is Relation.ENCLOSED
@@ -119,9 +122,9 @@ def relate_contour(polygon: Polygon, contour: Contour) -> Relation:
 
 def relate_region(polygon: Polygon, region: Region) -> Relation:
     border, holes = polygon
-    region_bounding_box = bounding_box.from_iterable(region)
+    region_bounding_box = bounding.box_from_iterable(region)
     relation_with_border = relate_regions(border, region,
-                                          bounding_box.from_iterable(border),
+                                          bounding.box_from_iterable(border),
                                           region_bounding_box)
     if relation_with_border in (Relation.DISJOINT,
                                 Relation.TOUCH,
@@ -135,74 +138,104 @@ def relate_region(polygon: Polygon, region: Region) -> Relation:
                 if holes
                 else relation_with_border)
     else:
-        relation_with_holes = _relate_region_to_multiregion(
-                holes, region, region_bounding_box)
+        relation_with_holes = relate_region_to_regions(holes, region,
+                                                       region_bounding_box)
         if relation_with_holes is Relation.DISJOINT:
             return relation_with_border
-        elif relation_with_holes is Relation.WITHIN:
-            return Relation.DISJOINT
         elif relation_with_holes is Relation.TOUCH:
             return Relation.ENCLOSED
-        elif (relation_with_holes is Relation.OVERLAP
-              or relation_with_holes is Relation.COMPOSITE):
-            return Relation.OVERLAP
-        elif relation_with_holes in (Relation.ENCLOSED,
+        elif relation_with_holes in (Relation.EQUAL,
                                      Relation.COMPONENT,
-                                     Relation.EQUAL):
+                                     Relation.ENCLOSED):
             return Relation.TOUCH
+        elif relation_with_holes is Relation.WITHIN:
+            return Relation.DISJOINT
         else:
             return Relation.OVERLAP
 
 
 def relate_multiregion(polygon: Polygon, multiregion: Multiregion) -> Relation:
+    return (_relate_multiregion(polygon, multiregion)
+            if multiregion
+            else Relation.DISJOINT)
+
+
+def _relate_multiregion(polygon: Polygon,
+                        multiregion: Multiregion) -> Relation:
     border, holes = polygon
-    border_relation = relate_region_to_multiregion(multiregion, border)
+    border_bounding_box = bounding.box_from_iterable(border)
     if not holes:
-        return border_relation.complement
-    elif border_relation in (Relation.DISJOINT,
-                             Relation.TOUCH,
-                             Relation.OVERLAP):
-        return border_relation
-    elif border_relation in (Relation.EQUAL,
-                             Relation.COMPONENT,
-                             Relation.ENCLOSED):
-        return Relation.ENCLOSES
-    elif border_relation is Relation.WITHIN:
-        return Relation.COVER
-    else:
-        holes_relation = relate_multiregions(multiregion, holes)
-        if holes_relation is Relation.DISJOINT:
-            return border_relation.complement
-        elif holes_relation is Relation.COVER:
-            return Relation.DISJOINT
-        elif holes_relation in (Relation.ENCLOSES,
-                                Relation.COMPOSITE,
-                                Relation.EQUAL):
-            return Relation.TOUCH
-        elif (holes_relation is Relation.OVERLAP
-              or holes_relation is Relation.COMPONENT):
+        return relate_region_to_regions(multiregion, border,
+                                        border_bounding_box).complement
+    none_touch = True
+    subsets_regions_indices = []
+    for region_index, region in enumerate(multiregion):
+        region_relation = relate_regions(border, region, border_bounding_box,
+                                         bounding.box_from_iterable(region))
+        if region_relation is Relation.TOUCH:
+            if none_touch:
+                none_touch = False
+        elif region_relation in (Relation.OVERLAP,
+                                 Relation.COVER,
+                                 Relation.ENCLOSES):
+            return region_relation
+        elif (region_relation is Relation.COMPOSITE
+              or region_relation is Relation.EQUAL):
+            return Relation.ENCLOSES
+        elif region_relation is not Relation.DISJOINT:
+            if none_touch and (region_relation is Relation.ENCLOSED
+                               or region_relation is Relation.COMPONENT):
+                none_touch = False
+            subsets_regions_indices.append(region_index)
+    if subsets_regions_indices:
+        is_subset_of_border = len(subsets_regions_indices) == len(multiregion)
+        relation_with_holes = relate_multiregions(
+                holes,
+                multiregion
+                if is_subset_of_border
+                else [multiregion[index] for index in subsets_regions_indices])
+        if relation_with_holes is Relation.DISJOINT:
+            return ((Relation.WITHIN
+                     if none_touch
+                     else Relation.ENCLOSED)
+                    if is_subset_of_border
+                    else Relation.OVERLAP)
+        elif relation_with_holes is Relation.TOUCH:
+            return (Relation.ENCLOSED
+                    if is_subset_of_border
+                    else Relation.OVERLAP)
+        elif relation_with_holes is Relation.OVERLAP:
+            return relation_with_holes
+        elif relation_with_holes in (Relation.COVER,
+                                     Relation.ENCLOSES,
+                                     Relation.COMPOSITE):
             return Relation.OVERLAP
-        elif holes_relation is Relation.TOUCH:
-            return Relation.ENCLOSED
+        elif relation_with_holes is Relation.WITHIN:
+            return (Relation.DISJOINT
+                    if none_touch
+                    else Relation.TOUCH)
         else:
-            return Relation.OVERLAP
+            return Relation.TOUCH
+    else:
+        return (Relation.DISJOINT
+                if none_touch
+                else Relation.TOUCH)
 
 
 def relate_polygon(goal: Polygon, test: Polygon) -> Relation:
     goal_border, goal_holes = goal
     test_border, test_holes = test
-    return _relate_polygon(goal_border, goal_holes,
-                           bounding_box.from_iterable(goal_border),
-                           test_border, test_holes,
-                           bounding_box.from_iterable(test_border))
+    return _relate_polygon(goal_border, goal_holes, test_border, test_holes,
+                           bounding.box_from_iterable(goal_border),
+                           bounding.box_from_iterable(test_border))
 
 
 def _relate_polygon(goal_border: Region,
                     goal_holes: Multiregion,
-                    goal_border_bounding_box: bounding_box.BoundingBox,
                     test_border: Region,
                     test_holes: Multiregion,
-                    test_border_bounding_box: bounding_box.BoundingBox
+                    goal_border_bounding_box: bounding.Box,
+                    test_border_bounding_box: bounding.Box
                     ) -> Relation:
     borders_relation = relate_regions(goal_border, test_border,
                                       goal_border_bounding_box,
@@ -218,12 +251,14 @@ def _relate_polygon(goal_border: Region,
                                   Relation.TOUCH,
                                   Relation.OVERLAP):
                 return Relation.OVERLAP
-            elif holes_relation is Relation.COVER:
+            elif holes_relation in (Relation.COVER,
+                                    Relation.ENCLOSES,
+                                    Relation.COMPOSITE):
                 return Relation.ENCLOSES
-            elif holes_relation is Relation.WITHIN:
-                return Relation.ENCLOSED
+            elif holes_relation is Relation.EQUAL:
+                return borders_relation
             else:
-                return holes_relation
+                return Relation.ENCLOSED
         else:
             return (Relation.ENCLOSES
                     if goal_holes
@@ -239,7 +274,7 @@ def _relate_polygon(goal_border: Region,
             for hole_index, hole in enumerate(goal_holes):
                 hole_relation = relate_regions(
                         test_border, hole, test_border_bounding_box,
-                        bounding_box.from_iterable(hole))
+                        bounding.box_from_iterable(hole))
                 if hole_relation is Relation.TOUCH:
                     if none_touch:
                         none_touch = False
@@ -256,7 +291,10 @@ def _relate_polygon(goal_border: Region,
             if subsets_holes_indices:
                 holes_relation = relate_multiregions(
                         test_holes,
-                        [goal_holes[index] for index in subsets_holes_indices])
+                        goal_holes
+                        if len(subsets_holes_indices) == len(goal_holes)
+                        else [goal_holes[index]
+                              for index in subsets_holes_indices])
                 if holes_relation is Relation.EQUAL:
                     return (Relation.ENCLOSED
                             if borders_relation is Relation.WITHIN
@@ -269,9 +307,9 @@ def _relate_polygon(goal_border: Region,
                 else:
                     return Relation.OVERLAP
             else:
-                return (Relation.DISJOINT
+                return (borders_relation
                         if none_touch
-                        else Relation.TOUCH)
+                        else Relation.ENCLOSED)
         else:
             return (Relation.ENCLOSED
                     if test_holes and borders_relation is Relation.COMPONENT
@@ -282,7 +320,7 @@ def _relate_polygon(goal_border: Region,
         for hole_index, hole in enumerate(test_holes):
             hole_relation = relate_regions(
                     goal_border, hole, goal_border_bounding_box,
-                    bounding_box.from_iterable(hole))
+                    bounding.box_from_iterable(hole))
             if hole_relation is Relation.TOUCH:
                 if none_touch:
                     none_touch = False
@@ -299,7 +337,10 @@ def _relate_polygon(goal_border: Region,
         if subsets_holes_indices:
             holes_relation = relate_multiregions(
                     goal_holes,
-                    [test_holes[index] for index in subsets_holes_indices])
+                    test_holes
+                    if len(subsets_holes_indices) == len(test_holes)
+                    else [test_holes[index]
+                          for index in subsets_holes_indices])
             if holes_relation is Relation.EQUAL:
                 return (Relation.ENCLOSES
                         if borders_relation is Relation.COVER
@@ -312,10 +353,16 @@ def _relate_polygon(goal_border: Region,
             else:
                 return Relation.OVERLAP
         else:
-            return (Relation.DISJOINT
+            return (borders_relation
                     if none_touch
-                    else Relation.TOUCH)
+                    else Relation.ENCLOSES)
     else:
         return (Relation.ENCLOSES
                 if goal_holes and borders_relation is Relation.COMPOSITE
                 else borders_relation)
+
+
+def to_segments(polygon: Polygon) -> Iterable[Segment]:
+    border, holes = polygon
+    yield from region_to_segments(border)
+    yield from multiregion_to_segments(holes)
