@@ -6,10 +6,11 @@ from robust.linear import (SegmentsRelationship,
                            segments_relationship)
 
 from orient.hints import (Contour,
+                          Multisegment,
                           Point,
                           Region,
                           Segment)
-from . import bounding_box
+from . import bounding
 from .contour import (equal as contours_equal,
                       point_vertex_line_divides_angle,
                       to_segments as contour_to_segments)
@@ -18,8 +19,7 @@ from .processing import (process_compound_queue,
 from .relation import Relation
 from .segment import (relate_point as relate_point_to_segment,
                       relate_segment as relate_segments)
-from .sweep import ClosedSweeper
-from .utils import flatten
+from .sweep import CompoundSweeper
 
 
 def relate_point(region: Region, point: Point) -> Relation:
@@ -261,16 +261,23 @@ def relate_segment(region: Region, segment: Segment) -> Relation:
                     else Relation.TOUCH)
 
 
-def relate_multisegment(region: Region, multisegment: Contour) -> Relation:
-    if not multisegment:
-        return Relation.DISJOINT
-    multisegment_bounding_box, region_bounding_box = (
-        bounding_box.from_points(flatten(multisegment)),
-        bounding_box.from_points(region))
-    if bounding_box.disjoint_with(multisegment_bounding_box,
+def relate_multisegment(region: Region,
+                        multisegment: Multisegment) -> Relation:
+    return (_relate_multisegment(region, multisegment,
+                                 bounding.box_from_iterables(multisegment))
+            if multisegment
+            else Relation.DISJOINT)
+
+
+def _relate_multisegment(region: Region,
+                         multisegment: Multisegment,
+                         multisegment_bounding_box: bounding.Box
+                         ) -> Relation:
+    region_bounding_box = bounding.box_from_iterable(region)
+    if bounding.box_disjoint_with(multisegment_bounding_box,
                                   region_bounding_box):
         return Relation.DISJOINT
-    sweeper = ClosedSweeper()
+    sweeper = CompoundSweeper()
     sweeper.register_segments(to_segments(region),
                               from_test=False)
     sweeper.register_segments(multisegment,
@@ -282,13 +289,20 @@ def relate_multisegment(region: Region, multisegment: Contour) -> Relation:
 
 
 def relate_contour(region: Region, contour: Contour) -> Relation:
-    contour_bounding_box, region_bounding_box = (
-        bounding_box.from_points(contour), bounding_box.from_points(region))
-    if bounding_box.disjoint_with(contour_bounding_box, region_bounding_box):
+    return _relate_contour(region, contour,
+                           bounding.box_from_iterable(contour))
+
+
+def _relate_contour(region: Region,
+                    contour: Contour,
+                    contour_bounding_box: bounding.Box
+                    ) -> Relation:
+    region_bounding_box = bounding.box_from_iterable(region)
+    if bounding.box_disjoint_with(contour_bounding_box, region_bounding_box):
         return Relation.DISJOINT
     if equal(region, contour):
         return Relation.COMPONENT
-    sweeper = ClosedSweeper()
+    sweeper = CompoundSweeper()
     sweeper.register_segments(to_segments(region),
                               from_test=False)
     sweeper.register_segments(contour_to_segments(contour),
@@ -300,13 +314,19 @@ def relate_contour(region: Region, contour: Contour) -> Relation:
 
 
 def relate_region(goal: Region, test: Region) -> Relation:
-    goal_bounding_box, test_bounding_box = (bounding_box.from_points(goal),
-                                            bounding_box.from_points(test))
-    if bounding_box.disjoint_with(goal_bounding_box, test_bounding_box):
+    return _relate_region(goal, test, bounding.box_from_iterable(goal),
+                          bounding.box_from_iterable(test))
+
+
+def _relate_region(goal: Region,
+                   test: Region,
+                   goal_bounding_box: bounding.Box,
+                   test_bounding_box: bounding.Box) -> Relation:
+    if bounding.box_disjoint_with(goal_bounding_box, test_bounding_box):
         return Relation.DISJOINT
     if equal(goal, test):
         return Relation.EQUAL
-    sweeper = ClosedSweeper()
+    sweeper = CompoundSweeper()
     sweeper.register_segments(to_segments(goal),
                               from_test=False)
     sweeper.register_segments(to_segments(test),
